==> machines-1.2.2.tgz: package/createTransitionHandler.js <==
"use strict";

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; var ownKeys = Object.keys(source); if (typeof Object.getOwnPropertySymbols === 'function') { ownKeys = ownKeys.concat(Object.getOwnPropertySymbols(source).filter(function (sym) { return Object.getOwnPropertyDescriptor(source, sym).enumerable; })); } ownKeys.forEach(function (key) { _defineProperty(target, key, source[key]); }); } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function _inheritsLoose(subClass, superClass) { subClass.prototype = Object.create(superClass.prototype); subClass.prototype.constructor = subClass; subClass.__proto__ = superClass; }

function _wrapNativeSuper(Class) { var _cache = typeof Map === "function" ? new Map() : undefined; _wrapNativeSuper = function _wrapNativeSuper(Class) { if (Class === null || !_isNativeFunction(Class)) return Class; if (typeof Class !== "function") { throw new TypeError("Super expression must either be null or a function"); } if (typeof _cache !== "undefined") { if (_cache.has(Class)) return _cache.get(Class); _cache.set(Class, Wrapper); } function Wrapper() { return _construct(Class, arguments, _getPrototypeOf(this).constructor); } Wrapper.prototype = Object.create(Class.prototype, { constructor: { value: Wrapper, enumerable: false, writable: true, configurable: true } }); return _setPrototypeOf(Wrapper, Class); }; return _wrapNativeSuper(Class); }

function isNativeReflectConstruct() { if (typeof Reflect === "undefined" || !Reflect.construct) return false; if (Reflect.construct.sham) return false; if (typeof Proxy === "function") return true; try { Date.prototype.toString.call(Reflect.construct(Date, [], function () {})); return true; } catch (e) { return false; } }

function _construct(Parent, args, Class) { if (isNativeReflectConstruct()) { _construct = Reflect.construct; } else { _construct = function _construct(Parent, args, Class) { var a = [null]; a.push.apply(a, args); var Constructor = Function.bind.apply(Parent, a); var instance = new Constructor(); if (Class) _setPrototypeOf(instance, Class.prototype); return instance; }; } return _construct.apply(null, arguments); }

function _isNativeFunction(fn) { return Function.toString.call(fn).indexOf("[native code]") !== -1; }

function _setPrototypeOf(o, p) { _setPrototypeOf = Object.setPrototypeOf || function _setPrototypeOf(o, p) { o.__proto__ = p; return o; }; return _setPrototypeOf(o, p); }

function _getPrototypeOf(o) { _getPrototypeOf = Object.setPrototypeOf ? Object.getPrototypeOf : function _getPrototypeOf(o) { return o.__proto__ || Object.getPrototypeOf(o); }; return _getPrototypeOf(o); }

var createMachine = require('./createMachine');

var _require = require('./_internal'),
    validateStateMachine = _require.validateStateMachine;
/**
 * Exends the base JavaScript base `Error`
 * @typedef {Object<string, any>} TransitionError
 * @property {string} name The name of the error (in this case "TransitionError")
 * @property {string} data.currentState The current state of the state machine at the time the error was encountered
 */


var TransitionError =
/*#__PURE__*/
function (_Error) {
  _inheritsLoose(TransitionError, _Error);

  function TransitionError(message, state, additionalData) {
    var _this;

    if (additionalData === void 0) {
      additionalData = {};
    }

    _this = _Error.call(this, message) || this;
    _this.name = _this.constructor.name;
    _this.data = _objectSpread({}, typeof additionalData === 'object' && additionalData.constructor.name === 'Object' ? additionalData : {}, {
      currentState: state
    });
    return _this;
  }

  return TransitionError;
}(_wrapNativeSuper(Error));
/**
 * Takes a given state machine and handler and returns a function which is ready to receive the starting "state" value _and_ any starting data, following through the entire sequence of async function until the state stops advancing.
 * Note, a handler will receive the initial data, the starting "state" value and any (optional) dependencies to place into the context for the async functions contained in the handler.
 * A handler doesn't have to be a function but can also be an object whose keys are possible "state" values and whose values are functions that receive the initial data, state machine, and context.
 *
 * @function
 * @name createTransitionHandler
 * @throws {TypeError} If a state machine is not provided
 * @throws {TypeError} If a handler function/object is not provided
 * @throws {TypeError} If the handler is not a function or an object whose keys do not correspond to keys in the state `machine`
 * @throws {TypeError} If the state machine has any values which are not objects
 * @throws {TypeError} If the state machine's "sub-objects" has values that are not keys on the root of the object
 * @param {function|Object<string, function>} handler A handler function or an object of individual handler function (the keys on the object must match possible "states" otherwise they'll never be called)
 * @param {Object<string, Object<string, string>>} machine An object whose keys are possible states and whose values are also objects (but whose keys are transition names and whose values are states they will advance to)
 * @returns {function} A wrapped handler function that is ready to receive the (1) initial data, (2) starting "state" value on the machine, and (3) an object containing any dependencies the handler may have
 */


function createTransitionHandler(handler, machine) {
  if (!machine) {
    throw new TypeError('A state machine is required but was not provided');
  }

  if (!handler) {
    throw new TypeError('A handler is required for the state machine transitions');
  }

  validateStateMachine(machine);
  var getStateHandler = typeof handler === 'function' ? function () {
    return handler;
  } : handler && typeof handler === 'object' && handler.constructor.name === 'Object' ? function (currentState) {
    if (typeof handler[currentState] !== 'function') {
      throw new TypeError("No handler was defined for the current state of '" + currentState + "'");
    }

    return handler[currentState];
  } : undefined;

  if (!getStateHandler) {
    throw new TypeError('handler must be a function or an object whose keys are state names and whose values are functions');
  }
  /**
   * A handler function that will advance from state to state - executing each piece of logic and/or async function the user defined, until the state can no longer advance and the current value or error is returned at that point.
   *
   * @function
   * @name wrappedTransitionHandler
   * @throws {TransitionError} If the handler encounters an unexpected exception
   * @param {Object<string, any>} initialData Any starting data for the handler
   * @param {string} initialState A starting "state" value which should be one of the possible values on the state machine
   * @param {Object<string, any>} [context] An optional object containing any dependencies of the handler (API clients, caches, etc.)
   * @returns {*} When the handler can no longer advance to another possible state, the value of the last function it executed is returned
   */


  return function wrappedTransitionHandler(initialData, initialState, context) {
    var currentState;
    var nextState = createMachine(machine, initialState || 'initial');

    function runNextTransition(r) {
      if (r && /Error$/.test(r.constructor.name)) {
        return Promise.reject(r);
      } else if (currentState !== nextState()) {
        currentState = nextState();
        var response = getStateHandler(currentState)(r, nextState, context);

        if (response != null && typeof response.then === 'function') {
          return response.then(function (res) {
            return runNextTransition(res);
          });
        } else {
          return runNextTransition(response);
        }
      } else {
        return Promise.resolve(r);
      }
    }

    return runNextTransition(initialData).catch(function (err) {
      throw new TransitionError(err.message || err.toString(), nextState(), _objectSpread({}, err.data && typeof err.data === 'object' || {}, err.extensions && typeof err.extensions === 'object' || {}));
    });
  };
}

module.exports = createTransitionHandler;

==> machines-1.2.3.tgz: package/createTransitionHandler.js <==
"use strict";

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; var ownKeys = Object.keys(source); if (typeof Object.getOwnPropertySymbols === 'function') { ownKeys = ownKeys.concat(Object.getOwnPropertySymbols(source).filter(function (sym) { return Object.getOwnPropertyDescriptor(source, sym).enumerable; })); } ownKeys.forEach(function (key) { _defineProperty(target, key, source[key]); }); } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function _inheritsLoose(subClass, superClass) { subClass.prototype = Object.create(superClass.prototype); subClass.prototype.constructor = subClass; subClass.__proto__ = superClass; }

function _wrapNativeSuper(Class) { var _cache = typeof Map === "function" ? new Map() : undefined; _wrapNativeSuper = function _wrapNativeSuper(Class) { if (Class === null || !_isNativeFunction(Class)) return Class; if (typeof Class !== "function") { throw new TypeError("Super expression must either be null or a function"); } if (typeof _cache !== "undefined") { if (_cache.has(Class)) return _cache.get(Class); _cache.set(Class, Wrapper); } function Wrapper() { return _construct(Class, arguments, _getPrototypeOf(this).constructor); } Wrapper.prototype = Object.create(Class.prototype, { constructor: { value: Wrapper, enumerable: false, writable: true, configurable: true } }); return _setPrototypeOf(Wrapper, Class); }; return _wrapNativeSuper(Class); }

function isNativeReflectConstruct() { if (typeof Reflect === "undefined" || !Reflect.construct) return false; if (Reflect.construct.sham) return false; if (typeof Proxy === "function") return true; try { Date.prototype.toString.call(Reflect.construct(Date, [], function () {})); return true; } catch (e) { return false; } }

function _construct(Parent, args, Class) { if (isNativeReflectConstruct()) { _construct = Reflect.construct; } else { _construct = function _construct(Parent, args, Class) { var a = [null]; a.push.apply(a, args); var Constructor = Function.bind.apply(Parent, a); var instance = new Constructor(); if (Class) _setPrototypeOf(instance, Class.prototype); return instance; }; } return _construct.apply(null, arguments); }

function _isNativeFunction(fn) { return Function.toString.call(fn).indexOf("[native code]") !== -1; }

function _setPrototypeOf(o, p) { _setPrototypeOf = Object.setPrototypeOf || function _setPrototypeOf(o, p) { o.__proto__ = p; return o; }; return _setPrototypeOf(o, p); }

function _getPrototypeOf(o) { _getPrototypeOf = Object.setPrototypeOf ? Object.getPrototypeOf : function _getPrototypeOf(o) { return o.__proto__ || Object.getPrototypeOf(o); }; return _getPrototypeOf(o); }

var createMachine = require('./createMachine');

var _require = require('./_internal'),
    validateStateMachine = _require.validateStateMachine;
/**
 * Exends the base JavaScript base `Error`
 * @typedef {Object<string, any>} TransitionError
 * @property {string} name The name of the error (in this case "TransitionError")
 * @property {string} data.currentState The current state of the state machine at the time the error was encountered
 */


var TransitionError =
/*#__PURE__*/
function (_Error) {
  _inheritsLoose(TransitionError, _Error);

  function TransitionError(message, state, additionalData) {
    var _this;

    if (additionalData === void 0) {
      additionalData = {};
    }

    _this = _Error.call(this, message) || this;
    _this.name = _this.constructor.name;
    _this.data = _objectSpread({}, typeof additionalData === 'object' && additionalData.constructor.name === 'Object' ? additionalData : {}, {
      currentState: state
    });
    return _this;
  }

  return TransitionError;
}(_wrapNativeSuper(Error));
/**
 * Takes a given state machine and handler and returns a function which is ready to receive the starting "state" value _and_ any starting data, following through the entire sequence of async function until the state stops advancing.
 * Note, a handler will receive the initial data, the starting "state" value and any (optional) dependencies to place into the context for the async functions contained in the handler.
 * A handler doesn't have to be a function but can also be an object whose keys are possible "state" values and whose values are functions that receive the initial data, state machine, and context.
 *
 * @function
 * @name createTransitionHandler
 * @throws {TypeError} If a state machine is not provided
 * @throws {TypeError} If a handler function/object is not provided
 * @throws {TypeError} If the handler is not a function or an object whose keys do not correspond to keys in the state `machine`
 * @throws {TypeError} If the state machine has any values which are not objects
 * @throws {TypeError} If the state machine's "sub-objects" has values that are not keys on the root of the object
 * @param {function|Object<string, function>} handler A handler function or an object of individual handler function (the keys on the object must match possible "states" otherwise they'll never be called)
 * @param {Object<string, Object<string, string>>} machine An object whose keys are possible states and whose values are also objects (but whose keys are transition names and whose values are states they will advance to)
 * @returns {function} A wrapped handler function that is ready to receive the (1) initial data, (2) starting "state" value on the machine, and (3) an object containing any dependencies the handler may have
 */


function createTransitionHandler(handler, machine) {
  if (!machine) {
    throw new TypeError('A state machine is required but was not provided');
  }

  if (!handler) {
    throw new TypeError('A handler is required for the state machine transitions');
  }

  validateStateMachine(machine);
  var getStateHandler = typeof handler === 'function' ? function () {
    return handler;
  } : handler && typeof handler === 'object' && handler.constructor.name === 'Object' ? function (currentState) {
    if (typeof handler[currentState] !== 'function') {
      throw new TypeError("No handler was defined for the current state of '" + currentState + "'");
    }

    return handler[currentState];
  } : undefined;

  if (!getStateHandler) {
    throw new TypeError('handler must be a function or an object whose keys are state names and whose values are functions');
  }
  /**
   * A handler function that will advance from state to state - executing each piece of logic and/or async function the user defined, until the state can no longer advance and the current value or error is returned at that point.
   *
   * @function
   * @name wrappedTransitionHandler
   * @throws {TransitionError} If the handler encounters an unexpected exception
   * @param {Object<string, any>} initialData Any starting data for the handler
   * @param {string} initialState A starting "state" value which should be one of the possible values on the state machine
   * @param {Object<string, any>} [context] An optional object containing any dependencies of the handler (API clients, caches, etc.)
   * @returns {*} When the handler can no longer advance to another possible state, the value of the last function it executed is returned
   */


  return function wrappedTransitionHandler(initialData, initialState, context) {
    var currentState;
    var nextState = createMachine(machine, initialState || 'initial');

    function runNextTransition(r) {
      if (r && /Error$/.test(r.constructor.name)) {
        return Promise.reject(r);
      } else if (currentState !== nextState()) {
        currentState = nextState();
        var response = getStateHandler(currentState)(r, nextState, context);

        if (response != null && typeof response.then === 'function') {
          return response.then(function (res) {
            return runNextTransition(res);
          });
        } else {
          return runNextTransition(response);
        }
      } else {
        return Promise.resolve(r);
      }
    }

    return runNextTransition(initialData).catch(function (err) {
      throw new TransitionError(err.message || err.toString(), nextState(), _objectSpread({}, err.data && typeof err.data === 'object' && err.data || {}, err.extensions && typeof err.extensions === 'object' && err.extensions || {}));
    });
  };
}

module.exports = createTransitionHandler;
